/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.ignite.schema.test.generator;

import java.io.File;
import org.apache.ignite.schema.generator.CodeGenerator;
import org.apache.ignite.schema.model.PojoDescriptor;
import org.apache.ignite.schema.test.AbstractSchemaImportTest;

/**
 * Tests for POJO generator.
 */
public class CodeGeneratorTest extends AbstractSchemaImportTest {
    /** Marker string to skip date generation while comparing.*/
    private static final String GEN_PTRN = "Code generated by Apache Ignite Schema Import utility";

    /**
     * Test that POJOs generated correctly.
     */
    public void testPojoGeneration() throws Exception {
        String pkg = "org.apache.ignite.schema.test.model";
        String intPath = "org/apache/ignite/schema/test/model";

        Boolean containsSchema = false;

        for (PojoDescriptor pojo : pojos) {
            if (pojo.valueClassName().isEmpty())
                containsSchema = true;
            else {
                CodeGenerator.pojos(pojo, OUT_DIR_PATH, pkg, true, true, askOverwrite);

                assertTrue("Generated key class POJO content is differ from expected for type " + pojo.keyClassName(),
                        compareFiles(pojo.keyClassName(), intPath, GEN_PTRN));

                assertTrue("Generated value class POJO content is differ from expected for type " + pojo.valueClassName(),
                        compareFiles(pojo.valueClassName(), intPath, GEN_PTRN));
            }
        }

        assertTrue("Generated POJOs does not contains schema.", containsSchema);
    }

    /**
     * @param typeName Type name.
     * @param intPath Internal path.
     * @return {@code true} if generated POJO as expected.
     */
    private boolean compareFiles(String typeName, String intPath, String excludePtrn) {
        String relPath = intPath + "/" + typeName;

        return compareFilesInt(getClass().getResourceAsStream("/" + relPath + ".txt"),
                new File(OUT_DIR_PATH + "/" + relPath + ".java"), excludePtrn);
    }
}